@charset "utf-8";

@-webkit-keyframes fade-in {
    from { opacity: 0; }
    to { opacity: 1; }
}

@keyframes fade-in {
    from { opacity: 0; }
    to { opacity: 1; }
}

@-webkit-keyframes fade-into-drop {
    from {
        opacity: 0;
        -o-opacity: 0;
        -ms-opacity: 0;
        -moz-opacity: 0;
        -khtml-opacity: 0;
        -webkit-opacity: 0;
        transform: translateY(-6px);
        -o-transform: translateY(-6px);
        -ms-transform: translateY(-6px);
        -moz-transform: translateY(-6px);
        -khtml-transform: translateY(-6px);
        -webkit-transform: translateY(-6px);
    }
    to {
        opacity: 1;
        -o-opacity: 1;
        -ms-opacity: 1;
        -moz-opacity: 1;
        -khtml-opacity: 1;
        -webkit-opacity: 1;
        transform: translateY(0px);
        -o-transform: translateY(0px);
        -ms-transform: translateY(0px);
        -moz-transform: translateY(0px);
        -khtml-transform: translateY(0px);
        -webkit-transform: translateY(0px);
    }
}

@keyframes fade-into-drop {
    from {
        opacity: 0;
        -o-opacity: 0;
        -ms-opacity: 0;
        -moz-opacity: 0;
        -khtml-opacity: 0;
        -webkit-opacity: 0;
        transform: translateY(-6px);
        -o-transform: translateY(-6px);
        -ms-transform: translateY(-6px);
        -moz-transform: translateY(-6px);
        -khtml-transform: translateY(-6px);
        -webkit-transform: translateY(-6px);
    }
    to {
        opacity: 1;
        -o-opacity: 1;
        -ms-opacity: 1;
        -moz-opacity: 1;
        -khtml-opacity: 1;
        -webkit-opacity: 1;
        transform: translateY(0px);
        -o-transform: translateY(0px);
        -ms-transform: translateY(0px);
        -moz-transform: translateY(0px);
        -khtml-transform: translateY(0px);
        -webkit-transform: translateY(0px);
    }
}

body {
    font-family: "Open Sans", "Lucida Grande", "Segoe UI", "Lucida Sans Unicode", "Geneva", "Verdana", sans-serif;
    font-size: 14px;
    margin: 0px auto 0px auto;
    padding: 0px 12px 0px 12px;
    word-break: break-all;
}

a {
    color: #214c8f;
    padding-bottom: 1px;
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}

.footer {
    border-top: 1px solid #eeeeee;
    font-size: 12px;
    line-height: 20px;
    margin: 10px 0px 20px 0px;
    padding: 10px 0px 0px 0px;
}

.message h1,
.debugger h1 {
    font-weight: normal;
    margin: 0px 0px 16px 0px;
}

.message h1.single,
.debugger h1.single {
    margin: 0px 0px 0px 0px;
}

.message p,
.debugger p {
    margin: 0px 0px 0px 0px;
}

.message .header,
.debugger .header {
    margin: 10px 0px 10px 0px;
    padding: 0px 0px 0px 0px;
}

.message .traceback,
.debugger .traceback {
    border-top: 1px solid #eeeeee;
    font-size: 13px;
    line-height: 24px;
    margin: 10px 0px 10px 0px;
    padding: 10px 0px 0px 0px;
}

.message .traceback .line.opener,
.debugger .traceback .line.opener {
    cursor: pointer;
}

.message .traceback .line.opener:hover,
.debugger .traceback .line.opener:hover {
    color: #0959d5;
    text-decoration: none;
}

.message .traceback .lines-extra,
.debugger .traceback .lines-extra {
    animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    -o-animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    -ms-animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    -moz-animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    -khtml-animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    -webkit-animation: fade-into-drop 0.25s cubic-bezier(0.645, 0.045, 0.355, 1.0);
    background-color: #f6f6f6;
    display: none;
    font-family: "SFMono-Regular", Consolas, "Liberation Mono", Menlo, Courier, monospace;
    font-size: 12px;
    line-height: 22px;
    margin: 8px 0px 8px 0px;
    overflow: auto;
    padding: 8px 0px 8px 0px;
}

.message .traceback .lines-extra .line:hover,
.debugger .traceback .lines-extra .line:hover {
    background-color: #e8e8e8;
}

.message .traceback .lines-extra .line.target,
.debugger .traceback .lines-extra .line.target {
    background-color: #e9eff9;
    font-weight: bold;
}

.message .traceback .lines-extra .line.target:hover,
.debugger .traceback .lines-extra .line.target:hover {
    background-color: #e8e8e8;
}

.message .traceback .lines-extra .line > .lineno,
.debugger .traceback .lines-extra .line > .lineno,
.message .traceback .lines-extra .line > .text,
.debugger .traceback .lines-extra .line > .text {
    display: block;
    min-height: 22px;
    vertical-align: middle;
}

.message .traceback .lines-extra .line > .lineno,
.debugger .traceback .lines-extra .line > .lineno {
    color: #7d7d7d;
    float: left;
    padding: 0px 12px 0px 0px;
    text-align: right;
    user-select: none;
    -o-user-select: none;
    -ms-user-select: none;
    -moz-user-select: none;
    -khtml-user-select: none;
    -webkit-user-select: none;
    vertical-align: middle;
    width: 40px;
}

.message .traceback .lines-extra .line > .text,
.debugger .traceback .lines-extra .line > .text {
    white-space: pre;
}

.message .traceback .lines-extra .line > .text:after,
.debugger .traceback .lines-extra .line > .text:after {
    clear: both;
    content: "";
}

.message .traceback .lines-extra .line > .text > *,
.debugger .traceback .lines-extra .line > .text > * {
    background-color: transparent;
}

.message .traceback .lines-extra .raw,
.debugger .traceback .lines-extra .raw {
    display: none;
}
